#------------------------------------------------------------------------
#
#   IBM Software
#   Licensed Material - Property of IBM
#   (C) Copyright International Business Machines Corp. 2007, 2017
#   All Rights Reserved.
#
#   U.S. Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  debug_utils.tcl
#
#  A collection of procedures that are useful for program analysis / debugging.
#------------------------------------------------------------------------

proc get_stack_trace { { cpu 0:0:0 } { sim mysim } } {

    # Handle both 32-bit and 64-bit ABI 
    if {[expr [$sim cpu $cpu display spr msr]>>63]} {
        # 64-bit ABI : pointer size is 8 bytes
        set PSZ 8
    } else {
        # 32-bit ABI : pointer size is 4 bytes
        set PSZ 4
    }

    set sp [ $sim cpu $cpu display gpr 1 ]
    set pc [ $sim cpu $cpu display spr pc ]

    set result ""

    set addr $pc
    while {$sp != 0} {

        set obj_files [object match_address_to_files $addr]
        set obj_file [lindex $obj_files 0]

        if {[string length $obj_file] > 0} {
            set addr_label [object label $obj_file $addr]
            if { [llength $addr_label] > 0 } {
                array set label_info $addr_label
                set source_label $label_info(LABEL)
            } else {
                set source_label  "????????"
            }

            # Might not have addr2line in the path, so catch potential error from this
            if {[catch {set source_line [exec addr2line -e $obj_file $addr]}]} {
                set source_line "????????"
            }
        } else {
            set source_label  "????????"
            set source_line "????????"
        }

        append result "{addr $addr label $source_label line $source_line} "

        # Link register is at +16 in stack frame
        if { [catch {set addr [ $sim cpu $cpu memory display [expr [$sim cpu $cpu dtranslate $sp] + ($PSZ*2)] $PSZ ]}] } {
            append result "{addr -1 label translate_fault line -1} "
            break
        }
        # Back chain is at +0 in stack frame
        if { [catch {set sp [ $sim cpu $cpu memory display [$sim cpu $cpu dtranslate $sp] $PSZ ]}] } {
            append result "{addr -1 label translate_fault line -1} "
            break
        }

        # Break if depth exceeds 50 to prevent infinite loop
        if {[llength $result] > 50} {
            break
        }
    }

    return $result
}

#------------------------------------------------------------------------
#   display_stack_trace -- display backtrace of stack of the process on the specified thread
#------------------------------------------------------------------------

proc display_stack_trace { { cpu 0:0:0 } { sim mysim } } {

    puts "Stack trace for CPU $cpu"

    set stack_trace [get_stack_trace $cpu $sim]

    for {set idx 0} {$idx < [llength $stack_trace]} {incr idx} {

        array set frame [lindex $stack_trace $idx]

        puts "\#$idx $frame(addr) in $frame(label) at $frame(line)"

    }
}

proc run { { sim mysim } } {
    $sim go
}
