#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2016, 2017
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  boot-linux-le-rtas.tcl
#
#  Boot Linux using mambo's built-in RTAS emulation. This way you
#  don't need firmware, just a vmlinux and a disk image.
#
#  NOTE: From a kernel perspective bare-metal RTAS like this isn't
#  actively supported, so it may break in later kernels; but it's working
#  right now (as of 4.8 being released).
#  For a more supported configuration you should look to the bare-metal
#  OPAL config in boot-linux-le-skiboot.tcl that uses skiboot.
#
#------------------------------------------------------------------------
# Run with:
#   /opt/ibm/systemsim-p9/run/p9/power9 -f ./boot-linux-le-rtas.tcl
#
# needs a little endian ./vmlinux and a ./disk.img
# 

source $env(LIB_DIR)/common/openfirmware_utils.tcl

set img_dir /opt/ibm/systemsim-p9/images/le

# Set the filename of the disk image and kernel here
set disk_img disk.img
set kernel vmlinux

# Increasing the number of hardware threads per processor or the number of
# CPUs in the simulated system will slow down your boot time because the
# underlying simulator is running on only a single thread on the host
# system
proc config_hook { conf } {
    $conf config cpus 1
    $conf config processor/number_of_threads 1
    $conf config memory_size 2048M

    # stop on attn instructions
    $conf config processor_option/ATTN_STOP true

    # have turbo mode handle mftb instructions
    $conf config turbo/mftb TRUE

    # Use an SMT4-style PIR with 2 bits for thread ID
    $conf config processor/PIR_bits_for_thread 2
    $conf config processor/PIR_bits_for_processor 9

    # We're not running under a hypervisor so we want external interrupts
    # directed to the SRRs rather than HSRRs, so we set LPCR[LPES(0)]=1
    $conf config processor/initial/LPCR 0x8
    # We're running a little endian kernel so we need to set the HILE bit
    # of HID so that we stay in little endian mode when handling interrupts
    $conf config processor/initial/HID 0x0800000000000000
}

proc clear_trigger_and_stop { sim mcm args } {
    # remove trigger
    array set triginfo $args
    set sim $triginfo(sim)
    $sim mcm $mcm trigger clear console $triginfo(match)

    simstop
}

proc run_until_shutdown_complete { { mcm 0 } { sim mysim } } {
    # catch the last message from the kernel after calling RTAS power-off
    $sim mcm $mcm trigger set console "RTAS power-off returned -1" { clear_trigger_and_stop $sim $mcm}
    $sim go
}

source $env(LIB_DIR)/p9/systemsim.tcl

source $env(RUN_DIR)/p9/p9-devtree.tcl

# Make sure the kernel can be found
if {[file exists ./$kernel]} {
    puts "Found kernel $kernel in current directory"
    set kernel_file ./$kernel
} elseif {[file exists $img_dir/$kernel]} {
    puts "Found kernel $kernel in $img_dir"
    set kernel_file $img_dir/$kernel
} else {
    puts "ERROR: Could not find kernel: $kernel"
    exit
}

# Make sure the disk image can be found
if {[file exists ./$disk_img]} {
    puts "Found disk image $disk_img in current directory"
    set diskimg_file ./$disk_img
} elseif {[file exists $img_dir/$disk_img]} {
    puts "Found disk image $disk_img in $img_dir"
    set diskimg_file $img_dir/$disk_img
} else {
    puts "ERROR: Could not find disk image: $disk_img"
    exit
}

# bogus disk
mysim bogus disk init 0 $diskimg_file rw
#mysim bogus disk init 0 $diskimg_file cow

# networking with IRQ off
# need this locally:
#   sudo tunctl -u $USER -t tap0
#   sudo ifconfig tap0 172.19.98.108 netmask 255.255.255.254
# in sim this:
#   ifconfig eth0 172.19.98.109 netmask 255.255.255.254
mysim bogus net init 0 d0:d0:d0:da:da:da tap0 0 0

# Load vmlinux
mysim load vmlinux $kernel_file 0x0

mysim mode fastest

# we could just mysim go, but let's be a bit smarter
# -> we'll catch the shutdown and stop there
run_until_shutdown_complete
#mysim go
