#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2016, 2017
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  boot-linux-le-skiboot.tcl
#
#  Boot Linux using real skiboot firmware.
#
#------------------------------------------------------------------------
# Run with:
#   /opt/ibm/systemsim-p9/run/p9/power9 -f ./boot-linux-le-skiboot.tcl
#
# needs a ./skiboot.lid, a little endian ./vmlinux, and a little endian
# ./disk.img
#

set img_dir /opt/ibm/systemsim-p9/images/le

# Set the filename of the skiboot lid, disk image and kernel here
set skiboot skiboot.lid
set disk_img disk.img
set kernel vmlinux

# location to load skiboot
set skiboot_addr 0

# location to load the kernel
set kernel_addr 0x20000000

# location to load the ePAPR flat devtree binary (place at 31M)
set epapr_dt_addr 0x1f00000

# value of PC after loading
set boot_pc 0x10

# Increasing the number of hardware threads per processor or the number of
# CPUs in the simulated system will slow down your boot time because the
# underlying simulator is running on only a single thread on the host
# system
proc config_hook { conf } {
    $conf config cpus 1
    $conf config processor/number_of_threads 1
    $conf config memory_size 2048M

    # Stop on attn instructions
    $conf config processor_option/ATTN_STOP true

    # We don't need these when we run with skiboot
    $conf config UART/0/enabled false
    $conf config SimpleUART/enabled false
    $conf config enable_rtas_support false
    $conf config enable_pseries_nvram false

    # We don't need the embedded-style RAM/ROM models
    $conf config machine_option/NO_RAM TRUE
    $conf config machine_option/NO_ROM TRUE

    # Have turbo mode handle mftb instructions
    $conf config turbo/mftb TRUE

    # Use an SMT4-style PIR with 2 bits for thread ID
    $conf config processor/PIR_bits_for_thread 2
    $conf config processor/PIR_bits_for_processor 9

    # Set a simulator speed limit to ensure we don't run too fast
    # when simulating idle periods
    $conf config max_simulator_speed_limit_default 4000000000
}

source $env(LIB_DIR)/p9/systemsim.tcl
source $env(RUN_DIR)/p9/p9-devtree-skiboot.tcl

source $env(LIB_DIR)/common/epapr.tcl
if {![info exists of::encode_compat]} {
    source $env(LIB_DIR)/common/openfirmware_utils.tcl
}

# Make sure skiboot can be found
if {[file exists ./$skiboot]} {
    puts "Found skiboot $skiboot in current directory"
    set skiboot_file ./$skiboot
} elseif {[file exists $img_dir/$skiboot]} {
    puts "Found skiboot $skiboot in $img_dir"
    set skiboot_file $img_dir/$skiboot
} else {
    puts "ERROR: Could not find skiboot: $skiboot"
    exit
}

# Make sure the kernel can be found
if {[file exists ./$kernel]} {
    puts "Found kernel $kernel in current directory"
    set kernel_file ./$kernel
} elseif {[file exists $img_dir/$kernel]} {
    puts "Found kernel $kernel in $img_dir"
    set kernel_file $img_dir/$kernel
} else {
    puts "ERROR: Could not find kernel: $kernel"
    exit
}

# Make sure the disk image can be found
if {[file exists ./$disk_img]} {
    puts "Found disk image $disk_img in current directory"
    set diskimg_file ./$disk_img
} elseif {[file exists $img_dir/$disk_img]} {
    puts "Found disk image $disk_img in $img_dir"
    set diskimg_file $img_dir/$disk_img
} else {
    puts "ERROR: Could not find disk image: $disk_img"
    exit
}

# bogus disk
mysim bogus disk init 0 $diskimg_file rw
#mysim bogus disk init 0 $diskimg_file newcow rootdisk.cow

# networking with IRQ off
# need this locally:
#   sudo tunctl -u $USER -t tap0
#   sudo ifconfig tap0 172.19.98.108 netmask 255.255.255.254
# in sim this:
#   ifconfig eth0 172.19.98.109 netmask 255.255.255.254
mysim bogus net init 0 d0:d0:d0:da:da:da tap0 0 0

# flatten the device tree
epapr::of2dtb mysim $epapr_dt_addr

# load images
set boot_size [file size $skiboot_file]
mysim memory fread $skiboot_addr $boot_size $skiboot_file

set payload_size [file size $kernel_file]
mysim memory fread $kernel_addr $payload_size $kernel_file

# init CPUs
set num_cores [ mysim mcm 0 display number_of_cpus ]
set num_threads_per_core [ expr [ mysim mcm 0 display number_of_threads ] / $num_cores ]
for { set coren 0 } { $coren < $num_cores } { incr coren } {
    for { set threadn 0 } { $threadn < $num_threads_per_core } { incr threadn } {
        mysim mcm 0 cpu $coren thread $threadn set spr pc $boot_pc
        mysim mcm 0 cpu $coren thread $threadn set gpr 3 $epapr_dt_addr
        mysim mcm 0 cpu $coren thread $threadn config_on
    }
}

puts "Booting with skiboot $skiboot_file....."
puts "Booting with kernel $kernel_file....."
puts "root disk $diskimg_file"

mysim mode fastest

mysim go
